<?php
class TagSuggest
{
    public $str;
    public $tokens;
    public $filteredTokens;
    public $excludeCount;
    public $suggestedTags;
    
    public $totalCount;
    public $minCount = 3;
    public $forEvery = 350;
    
    public $commonTokens = array(
        // articles
        'the', 'a',
        // prepositions
        'about', 'behind', 'from', 'on', 'toward', 'above', 'below', 
        'in', 'under', 'across', 'beneath', 'onto', 'underneath', 'after', 
        'beside', 'inside', 'until', 'against', 'between', 'outside', 'up', 
        'along', 'by', 'into', 'over', 'upon', 'among', 'down', 'like', 'past', 
        'with', 'around', 'during', 'near', 'since', 'within', 'at', 'except', 
        'of', 'through', 'without', 'before', 'for', 'off', 'to', 
        // incedental (3+ characters) (note: "each" must be excluded because of a bug in prototype.js.)
        'each', 'then', 'that', 'only', 'this', 'both', 'what', 'know', 'have', 
        'really', 'they', 'when', 'will', 'which', 'while', 'where', 'there', 
        'must', 'many', 'could', 'their', 'came', 'were', 'went', 'would', 
        'every', 'these', 'them', 'some', 'more', 'than', 'become', 'becomes', 
        'whether', 'here', 'instead', 'well', 'other', 'just', 'because', 
        'being', 'been', 'your', 'our', 'yours', 'less', 'most', 'said', 
        'comes', 'doing', 'also', 'ours', 'going', 'those', 'ever', 'piece', 
        'much', 'least', 'shall', 'simply', 'want', 'bring', 'might', 'may', 
        'does', 'sometimes', 'unto', 'thou', 'thee', 'hast', 'hath', 'shalt', 
        'come', 'even', 'often', 'very', 'always', 'including', 'and', 'was', 
        'his', 'but', 'who', 'did', 'get', 'him', 'out', 'all', 'her', 'his', 
        'hers', 'had', 'one', 'two', 'not', 'way', 'got', 'knew', 'rather', 
        'its', 'ibid', 'are', 'use', 'such', 'can', 'could', 'using', 'thus', 
        'you', 'yet', 'pre', 'likely', 'has', 'last', 'first', 'ways', 'why', 
        'how', 'didn', 'couldn', 'wouldn', 'shouldn', 'hadn', 'mustn'
    );
    
    public function __construct($str)
    {
        $this->str = $str;
        $this->tokenize();
        $this->filterTokens();
        $this->setExcludeCount();
        $this->suggestTags();
    }
    
    private function tokenize()
    {
        preg_match_all('/\b\w+\b/', $this->str, $tokens);
        $this->tokens = $tokens[0];
        $this->totalCount = count($tokens[0]);
    }
    
    private function filterTokens()
    {
        $filteredTokens = array();
        foreach ($this->tokens as $token) {
            $token = strtolower($token);
            if (!array_key_exists($token, $filteredTokens)) $filteredTokens[$token] = 0;
            $filteredTokens[$token]++;
        }
        $this->filteredTokens = $filteredTokens;
    }
    
    private function setExcludeCount()
    {
        // for every {forEvery} words the minimum word count should go up 1, starting at {minCount}
        /*
        $excludeCount = floor($this->totalCount / $this->forEvery);
        $this->excludeCount = $excludeCount > $this->minCount ? $excludeCount : $this->minCount;
        */
        
        // @todo: experiment using standard deviation.
        $deviation = $this->deviation($this->filteredTokens);
        $excludeCount = $deviation - $deviation/4;
        $this->excludeCount = $excludeCount > $this->minCount ? $excludeCount : $this->minCount;
    }
    
    private function suggestTags()
    {
        $suggestedTags = $this->filteredTokens;
        
        foreach ($suggestedTags as $token => $count) {
            // exclude common tokens
            if (in_array($token, $this->commonTokens)) unset($suggestedTags[$token]);
            
            // exclude tokens of 1 to 3 characters in length
            if (preg_match('/^\w{1,2}$/', $token)) unset($suggestedTags[$token]);
            
            // exclude tokens with a certain count
            if ($count < $this->excludeCount) unset($suggestedTags[$token]);
        }
        
        arsort($suggestedTags);
        $this->suggestedTags = $suggestedTags;
    }

    // @todo: experiment using standard deviation, mean, median, mode
    private function average($array) {
        $sum = array_sum($array);
        $count = count($array);
        return $sum/$count;
    }

    private function deviation($array) {
        $avg = $this->average($array);
        foreach ($array as $value) {
            $variance[] = pow($value-$avg, 2);
        }
        $deviation = sqrt($this->average($variance));
        return $deviation;
    }
}

include('Zend/Json.php');

$tagSuggest = new TagSuggest($_POST['str']);

$json = Zend_Json::encode($tagSuggest->suggestedTags);

header('X-JSON: (' . $json . ')');
?>